package com.link.core.basic.lock;

import com.link.base.user.model.CoreUser;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.cllog.LCLogger;
import com.link.core.parserplugin.AbstractScriptParser;
import com.link.core.parserplugin.ParserEnum;
import com.link.core.parserplugin.parser.JavaScriptParser;
import com.link.core.parserplugin.parser.OgnlParser;
import com.link.core.parserplugin.parser.SpringELParser;
import com.link.core.util.SpringUtil;
import com.link.core.util.StringUtils;
import com.link.core.util.UserUtil;
import java.lang.reflect.InvocationTargetException;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class LockHandler {
   Logger logger = LoggerFactory.getLogger(LockHandler.class);
   private final LockConfigure lockConfigure;

   public LockHandler(LockConfigure lockConfigure) {
      this.lockConfigure = lockConfigure;
   }

   public Object proceed(LockAopProxyChain pjp, LinkLock lock) throws Exception {
      RLock rLock = null;

      Object var19;
      try {
         CoreUser user = UserUtil.getUser();
         String keyEl = lock.key();
         Object[] arguments = pjp.getArgs();
         if (this.isLockable(lock, pjp.getTarget(), arguments)) {
            this.exceptionTypeHandler(lock, user, keyEl);
            RedissonClient redissonClient = (RedissonClient)SpringUtil.getBean(RedissonClient.class);
            rLock = redissonClient.getLock(this.getKey(lock, keyEl, pjp.getTarget(), arguments) + UserUtil.getUser().getOpenId());
            if (rLock.isLocked()) {
               throw new BasicServiceException(lock.errorMsg());
            }

            boolean lockFlag = rLock.tryLock((long)lock.waitTime(), (long)lock.expire(), TimeUnit.SECONDS);
            if (!lockFlag) {
               throw new BasicServiceException(lock.errorMsg());
            }
         }

         var19 = pjp.doProxyChain(arguments);
      } catch (InvocationTargetException var13) {
         InvocationTargetException e = var13;
         Throwable targetException = e.getTargetException();
         LCLogger.withException(targetException);
         throw new Exception(targetException.getMessage());
      } catch (Exception var14) {
         Exception e = var14;
         throw e;
      } finally {
         if (rLock != null && rLock.isHeldByCurrentThread()) {
            rLock.unlock();
         }

      }

      return var19;
   }

   private void exceptionTypeHandler(LinkLock lock, CoreUser user, String key) {
      if (!ClinetType.NO_LIMIT.equals(lock.client())) {
         Boolean checkLogin = !ClinetType.NO_LOGIN.equals(lock.client()) && Objects.isNull(user);
         Boolean checkMp = ClinetType.MP.equals(lock.client()) && StringUtils.isBlank(user.getOpenId());
         Boolean checkCrm = ClinetType.CRM.equals(lock.client()) && StringUtils.isBlank(user.getId());
         if (checkLogin || checkMp || checkCrm) {
            if (ExceptionType.THROW.equals(lock.exceptionType())) {
               throw new RuntimeException("服务异常，请联系管理员检查");
            }

            if (ExceptionType.LOGGER.equals(lock.exceptionType())) {
               this.logger.error("获取锁异常");
            }
         }

      }
   }

   private AbstractScriptParser getParser() {
      if (ParserEnum.SPRING_EL.getType().equals(this.lockConfigure.getParser().getType())) {
         return new SpringELParser();
      } else {
         return (AbstractScriptParser)(ParserEnum.JAVA_SCRIPT.getType().equals(this.lockConfigure.getParser().getType()) ? new JavaScriptParser() : new OgnlParser());
      }
   }

   private String getKey(LinkLock lock, String keyEl, Object target, Object[] arguments) throws Exception {
      CoreUser user = UserUtil.getUser();
      boolean hasUserVal = Objects.nonNull(user);
      String key = this.getParser().getStringElValue(keyEl, target, arguments, (Object)null, false, user, hasUserVal);
      if (ClinetType.MP.equals(lock.client())) {
         key = key + UserUtil.getUser().getOpenId();
      } else if (ClinetType.CRM.equals(lock.client())) {
         key = key + UserUtil.getUser().getId();
      }

      this.logger.info("全局性分布式锁key：" + key);
      return key;
   }

   public boolean isLockable(LinkLock lock, Object target, Object[] arguments) throws Exception {
      boolean rv = true;
      if (null != arguments && arguments.length > 0 && null != lock.condition() && lock.condition().length() > 0) {
         CoreUser user = UserUtil.getUser();
         boolean hasUserVal = Objects.nonNull(user);
         rv = this.getParser().getBooleanElValue(lock.condition(), target, arguments, (Object)null, false, user, hasUserVal);
      }

      this.logger.info("是否需要锁判断结果：" + rv);
      return rv;
   }
}
